# Generated by cpp11: do not edit by hand

bq_parse <- function(meta_s, data_s) {
  .Call(`_bigrquery_bq_parse`, meta_s, data_s)
}

bq_field_init <- function(json, value) {
  .Call(`_bigrquery_bq_field_init`, json, value)
}

bq_parse_files <- function(schema_path, file_paths, n, quiet) {
  .Call(`_bigrquery_bq_parse_files`, schema_path, file_paths, n, quiet)
}
