# bq_perform_upload creates job that succeeds

    Code
      bq_job_wait(job, quiet = FALSE)
    Message
      Job complete
      Input: <xxx> kB
      Output: <xxx> kB
    Code
      bq_job_wait(job, quiet = FALSE)
    Message
      Job complete
      Input: <xxx> kB
      Output: <xxx> kB

# bq_perform_copy creates job that succeeds

    Code
      bq_job_wait(job, quiet = FALSE)
    Message
      Job complete

# bq_perform_query creates job that succeeds

    Code
      bq_job_wait(job, quiet = FALSE)
    Message
      Job complete
      Billed: 0 B

