% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bq-auth.R
\name{bq_user}
\alias{bq_user}
\title{Get info on current user}
\usage{
bq_user()
}
\value{
An email address or, if no token has been loaded, \code{NULL}.
}
\description{
Reveals the email address of the user associated with the current token.
If no token has been loaded yet, this function does not initiate auth.
}
\examples{
\dontrun{
bq_user()
}
}
\seealso{
\code{\link[gargle:token_userinfo]{gargle::token_userinfo()}}, \code{\link[gargle:token_email]{gargle::token_email()}},
\code{\link[gargle:token_tokeninfo]{gargle::token_tokeninfo()}}
}
